window.fc = window.fc || {};
window.fc.platform = window.fc.platform || {};

(function (exports, Laya) {
    'use strict';

    var Event = fc.define.Event;
    var SceneId = fc.define.SceneId;
    var Audio = fc.framework.Audio;
    var GameFrame = fc.framework.GameFrame;
    var Channel = fc.framework.Channel;
    var BannerCtrl = fc.plugin.BannerCtrl;
    var BlockAdCtrl = fc.plugin.BlockAdCtrl;
    var AdditiveUICtrl = fc.plugin.AdditiveUICtrl;
    var ActivityCenter = fc.view.ActivityCenter;

    var viewCfg = {
        "Award": {
            "url": "fc-layout/view/AwardPage.scene",
            "script": "fc.view.AwardPage"
        },
        "Loading": {
            "url": "fc-layout/view/LoadingPage.scene",
            "script": "fc.view.LoadingPage"
        },
        "Home": {
            "url": "fc-layout/view/HomePage.scene",
            "script": "fc.view.HomePage"
        },
        "Game": {
            "url": "fc-layout/view/GamePage.scene",
            "script": "fc.view.GamePage"
        },
        "Success": {
            "url": "fc-layout/view/SuccessPage.scene",
            "script": "fc.view.SuccessPage"
        },
        "Failed": {
            "url": "fc-layout/view/FailedPage.scene",
            "script": "fc.view.FailedPage"
        },
        "AdHome": {
            "url": "fc-layout/view/AdHomePage.scene",
            "script": "fc.view.AdHomePage"
        },
        "MoreInfo": {
            "url": "fc-layout/view/MoreInfoPage.scene",
            "script": "fc.view.MoreInfoPage"
        },
        "Knock": {
            "url": "fc-layout/view/KnockPage.scene",
            "script": "fc.view.KnockPage"
        },
        "Sign": {
            "url": "fc-layout/view/SignPage.scene",
            "script": "fc.view.SignPage"
        },
        "Store": {
            "url": "fc-layout/view/StorePage.scene",
            "script": "fc.view.StorePage"
        },
        "Strength": {
            "url": "fc-layout/view/StrengthPage.scene",
            "script": "fc.view.StrengthPage"
        },
        "SecretBox": {
            "url": "fc-layout/view/SecretBoxPage.scene",
            "script": "fc.view.SecretBoxPage"
        },
        "SecretAward": {
            "url": "fc-layout/view/SecretAwardPageOppo.scene",
            "script": "fc.view.SecretAwardPage"
        },
        "ShareRec": {
            "url": "fc-layout/view/ShareRecPage.scene",
            "script": "fc.view.ShareRecPage"
        },
        "Trial": {
            "url": "fc-layout/view/TrialPageOppo.scene",
            "script": "fc.view.TrialPage"
        },
        "Turntable": {
            "url": "fc-layout/view/TurntablePage.scene",
            "script": "fc.view.TurntablePage"
        },
        "InsertAd": {
            "url": "fc-layout/view/VideoPage.scene",
            "script": "fc.view.InsertAdPage"
        },
        "FullScreenAd": {
            "url": "fc-layout/view/VideoPage.scene",
            "script": "fc.view.FullScreenAdPage"
        },
        "NativeAdbOppo": {
            "url": "fc-layout/view/NativeAdbPageOppo.scene",
            "script": "fc.view.NativeAdPage"
        },
        "NativeAdb": {
            "url": "fc-layout/view/NativeAdbPage.scene",
            "script": "fc.view.NativeAdPage"
        },
        "NativeAdm": {
            "url": "fc-layout/view/NativeAdmPage.scene",
            "script": "fc.view.NativeAdPage"
        },
        "NativeAdn": {
            "url": "fc-layout/view/NativeAdnPage.scene",
            "script": "fc.view.NativeAdPage"
        },
        "NativeBannerAd": {
            "url": "fc-layout/view/NativeAdBannerPage.scene",
            "script": "fc.view.NativeBannerAdPage"
        }
    };
    var sceneCfg = {
        "loading": {
            "type": "sync",
            "array": [
                {
                    "viewId": "Loading"
                }
            ]
        },
        "home": {
            "type": "sync",
            "array": [
                {
                    "viewId": "Home",
                    "zOrder": 1
                },
                {
                    "viewId": "Sign",
                    "checkCode": 1,
                    "zOrder": 2
                },
                {
                    "viewId": "Store",
                    "checkCode": 2,
                    "zOrder": 2,
                    "delay": 200
                }
            ]
        },
        "prepare": {
            "type": "sync",
            "array": [
                {
                    "viewId": "NativeAdbOppo",
                    "checkCode": 3,
                    "zOrder": 1
                },
                {
                    "viewId": "Trial",
                    "checkCode": 3,
                    "zOrder": 0
                }
            ]
        },
        "game": {
            "type": "sync",
            "array": [
                {
                    "viewId": "Game"
                }
            ]
        },
        "showSuccess": {
            "type": "sync",
            "array": [
                {
                    "viewId": "NativeAdbOppo",
                    "checkCode": 7,
                    "zOrder": 1
                },
                {
                    "viewId": "SecretAward",
                    "checkCode": 7,
                    "zOrder": 0,
                    "delay": 1000
                }
            ]
        },
        "showFailed": {
            "type": "sync",
            "array": [
                {
                    "viewId": "NativeAdb",
                    "checkCode": 7,
                    "zOrder": 1
                },
                {
                    "viewId": "SecretBox",
                    "checkCode": 7,
                    "zOrder": 0,
                    "delay": 1000
                }
            ]
        },
        "success": {
            "type": "sequence",
            "array": [
                {
                    "viewId": "Success"
                }
            ]
        },
        "failed": {
            "type": "sequence",
            "array": [
                {
                    "viewId": "Failed"
                }
            ]
        },
        "over": {
            "type": "sequence",
            "array": [
            ]
        }
    };   
    var blockConfig = {
        "Failed": {"x": 0, "y": 270, "size": 5,"delay": 10},
        "Success": {"x": 0, "y": 270, "size": 5,"delay": 10}
    } 
    var bannerConfig = {
        "Home": {"delay": 10},
    };
    var additiveUIConfig = {
        "Home": [
            {
                "viewUrl": "fc-layout/prefab/additive/GoldShowAdd.json",
                "parentRoot":"topUI",
                "x":32,
                "y":60,
                "scriptType":"fc.view.GoldShowAdd"
            },
            {
                "viewUrl": "fc-layout/prefab/additive/StrengthShowAdd.json",
                "parentRoot":"topUI",
                "x":32,
                "y":150,
                "scriptType":"fc.view.StrengthShowAdd",
                "checkCode": 2
            },
            {
                "viewUrl": "fc-layout/prefab/additive/MoreRecommendBtn.json",
                "parentRoot":"middleUI",
                "x":42,
                "y":331,
                "scriptType":"fc.view.MoreGameBoxBtn"
            },
            {
                "viewUrl": "fc-layout/prefab/additive/StartGame.json",
                "parentRoot":"bottomUI",
                "x":375,
                "y":359,
                "scriptType":"fc.view.AniLoopPlay"
            }
           
        ],    
        "Game": [
            {
                "viewUrl": "fc-layout/prefab/additive/GoldShow.json",
                "parentRoot":"topUI",
                "x":32,
                "y":60,
                "scriptType":"fc.view.GoldShow"
            },
            {
                "viewUrl": "fc-layout/prefab/additive/StrengthShow.json",
                "parentRoot":"topUI",
                "x":32,
                "y":150,
                "scriptType":"fc.view.StrengthShow",
                "checkCode": 2
            },
            {
                "viewUrl": "fc-layout/prefab/additive/ResetBtn.json",
                "parentRoot":"topUI",
                "x":30,
                "y":420,
                "scriptType":"fc.view.ResetBtn",
                "checkCode": 3
            }
        ],
        "Failed": [
            {
                "viewUrl": "fc-layout/prefab/additive/FailEmoji.json",
                "parentRoot": "topUI",
                "x": 375,
                "y": 370,
                "scriptType": "fc.view.AniLoopPlay"
            }
        ], 
        "Success": [
            {
                "viewUrl": "fc-layout/prefab/additive/SuccessEmoji.json",
                "parentRoot": "topUI",
                "x": 375,
                "y": 370,
                "scriptType": "fc.view.AniLoopPlay"
            }
        ]
    };
    var playerCfg = {
        "userId": "1",
        "isNew": 1, 
        "isInGuide": 1,
        "signedDays": 0, 
        "signTime": 0,
        "levelId": 1,
        "gold": 0,
        "strength": 5,
        "sound": 1,
        "goodsInUse": [0],
        "goodsInBag": [[0]]
    };
    
    function isAdditiveUIVisible(code) {

        if (!code) {
            return true;
        }

        if (code == 1) {
            return ActivityCenter.isEnableExportAd();
        }

        if (code == 2) {
            return ActivityCenter.isStrengthOpened();
        }

        if (code == 3) {
            return ActivityCenter.isResetGameOpened();
        }

        if (code == 4) {
            return ActivityCenter.isNativeActive();
        }

        return false;
    }

    function isViewValid(code) {
        if (!code) {
            return true;
        }

        if (code == 1) {
            return ActivityCenter.isEnablePopSign();
        }

        if (code == 2) {
            return ActivityCenter.isEnablePopStore();
        }

        if (code == 3) {
            return ActivityCenter.isEnablePopTrial();
        }

        if (code == 4) {
            return ActivityCenter.isEnablePopHomeAd();
        }

        if (code == 7) {
            return ActivityCenter.isEnablePopSecretAward();
        }

        if (code == 104) {
            return ActivityCenter.isEnablePopVip();
        }

        return false;
    }

    class PlatformConfig extends Laya.Script {

        static get playerInfo() {
            return GameFrame.playerInfo;
        }

        static set playerInfo(val) {
            if (val) {
                var keys = Object.keys(val);
                for (var index = 0; index < keys.length; index++) {
                    GameFrame.playerInfo[keys[index]] = val[keys[index]];
                }
                if (keys.length > 0) {
                    GameFrame.savePlayerInfo(val);
                }
            }
        }

        static init(appCfgUrl) {
            Laya.stage.addComponent(PlatformConfig);
            Laya.stage.addComponent(BannerCtrl);
            Laya.stage.addComponent(BlockAdCtrl);
            Laya.stage.addComponent(AdditiveUICtrl);
            Laya.stage.addComponent(ActivityCenter);
            GameFrame.startUp(appCfgUrl ? appCfgUrl : `config/${this.platformTag}/appConfig.${this.platformTag}.json`, PlatformConfig.platformTag, viewCfg, sceneCfg, playerCfg);
        }

        constructor() {
            super();
        }

        onAwake() {
            GameFrame.setViewValidCheckFun(isViewValid);
            BannerCtrl.bannerConfig = bannerConfig;
            BlockAdCtrl.blockConfig = blockConfig;
            AdditiveUICtrl.additiveUIConfig = additiveUIConfig;
            AdditiveUICtrl.checkFun = isAdditiveUIVisible;
            
            Laya.stage.once(Event.LAUNCH_COMPLETED, this, this.onLaunchResReady);
            Laya.stage.on(Event.UI_SCENE_LOADED, this, this.onSceneLoaded);
            Laya.stage.on(Event.UI_SCENE_UNLOADED, this, this.onSceneUnloaded);
            Laya.stage.on(Event.UI_VIEW_CLOSED, this, this.onViewClosed);
            Laya.stage.on(Event.UI_VIEW_OPENED, this, this.onViewOpened);
        }

        onDestroy() {
            Laya.stage.off(Event.UI_SCENE_LOADED, this, this.onSceneLoaded);
            Laya.stage.off(Event.UI_SCENE_UNLOADED, this, this.onSceneUnloaded);
            Laya.stage.off(Event.UI_VIEW_CLOSED, this, this.onViewClosed);
            Laya.stage.off(Event.UI_VIEW_OPENED, this, this.onViewOpened);
        }

        onLaunchResReady(scene3d) {
            var pluginCfg = GameFrame.pluginCfg;
            if (pluginCfg) {
                if (pluginCfg.pg_share_opened == null) {
                    pluginCfg.pg_share_opened = 0;
                }
                GameFrame.pluginCfg.pg_video_style = pluginCfg.pg_video_style || [0];
                GameFrame.pluginCfg.pg_ad_img_switch = pluginCfg.pg_ad_img_switch || 0;
            }
        }

        onSceneUnloaded(sceneId) {
            if (sceneId == SceneId.Game) {
                Audio.stopMusic();
            }
        }

        onSceneLoaded(sceneId) {
            if (sceneId == SceneId.Game) {
                if (GameFrame.appInfo.bgm) {
                    Audio.playMusic(GameFrame.appInfo.bgm);
                }
            }
        }

        onViewClosed(viewName, viewId) {
            if (viewId == "Failed" || viewId == "Success") {
                if (ActivityCenter.isMistakenActive()) {
                    // Channel.showGameBox(null, null, null);
                }
            }
        }

        onViewOpened(viewName, viewId) {
            console.log("onViewOpened", viewName);
    
            if (viewId == "Success" || viewId == "Failed" ) {
    
                if(fc.view.ActivityCenter.isMistakenActive() && (GameFrame.pluginCfg.pg_auto_showdesktop || 2) == fc.view.ActivityCenter.gameTime){
                    var handler = Laya.Handler.create(this, function (data) {
             
                        if(data && data.hasIcon == false){
                            fc.framework.Channel.createDesktopIcon(null);
                        }
                    });
                    fc.framework.Channel.hasDesktopIcon(handler);
                }
            }
        }
    }
    PlatformConfig.platformTag = "oppoymr";
    
    exports.PlatformConfig = PlatformConfig;
    
}(window.fc.platform, Laya));