window.fc = window.fc || {};
window.fc.platform = window.fc.platform || {};

(function (exports, Laya) {
    'use strict';

    var Event = fc.define.Event;
    var SceneId = fc.define.SceneId;
    var Audio = fc.framework.Audio;
    var GameFrame = fc.framework.GameFrame;
    var Channel = fc.framework.Channel;
    var BannerCtrl = fc.plugin.BannerCtrl;
    var AdditiveUICtrl = fc.plugin.AdditiveUICtrl;
    var ActivityCenter = fc.view.ActivityCenter;

    function isAdditiveUIVisible(code) {
        if (!code) {
            return true;
        }

        if (code == 2) {
            return ActivityCenter.isStrengthOpened();
        }

        if (code == 3) {
            return ActivityCenter.isResetGameOpened();
        }

        return false;
    }

    function isViewValid(code) {
        if (!code) {
            return true;
        }

        if (code == 1) {
            return ActivityCenter.isEnablePopSign();
        }

        if (code == 2) {
            return ActivityCenter.isEnablePopStore();
        }

        if (code == 3) {
            return ActivityCenter.isEnablePopInsertAd();
        }

        if (code == 4) {
            return ActivityCenter.isEnablePopTrial();
        }

        if (code == 104) {
            return ActivityCenter.isEnablePopVip();
        }

        return false;
    }
    
    var viewCfg = {
        "Award": {
            "url": "fc-layout/view/AwardPage.scene",
            "script": "fc.view.AwardPage"
        },
        "Loading": {
            "url": "fc-layout/view/LoadingPage.scene",
            "script": "fc.view.LoadingPage"
        },
        "Home": {
            "url": "fc-layout/view/HomePage.scene",
            "script": "fc.view.HomePage"
        },
        "Game": {
            "url": "fc-layout/view/GamePage.scene",
            "script": "fc.view.GamePage"
        },
        "Success": {
            "url": "fc-layout/view/SuccessPage.scene",
            "script": "fc.view.SuccessPage"
        },
        "Failed": {
            "url": "fc-layout/view/FailedPage.scene",
            "script": "fc.view.FailedPage"
        },
        "AdHome": {
            "url": "fc-layout/view/AdHomePage.scene",
            "script": "fc.view.AdHomePage"
        },
        "MoreInfo": {
            "url": "fc-layout/view/MoreInfoPage.scene",
            "script": "fc.view.MoreInfoPage"
        },
        "Knock": {
            "url": "fc-layout/view/KnockPage.scene",
            "script": "fc.view.KnockPage"
        },
        "Sign": {
            "url": "fc-layout/view/SignPage.scene",
            "script": "fc.view.SignPage"
        },
        "Store": {
            "url": "fc-layout/view/StorePage.scene",
            "script": "fc.view.StorePage"
        },
        "Strength": {
            "url": "fc-layout/view/StrengthPage.scene",
            "script": "fc.view.StrengthPage"
        },
        "SecretBox": {
            "url": "fc-layout/view/SecretBoxPage.scene",
            "script": "fc.view.SecretBoxPage"
        },
        "SecretAward": {
            "url": "fc-layout/view/SecretAwardPage.scene",
            "script": "fc.view.SecretAwardPage"
        },
        "ShareRec": {
            "url": "fc-layout/view/ShareRecPage.scene",
            "script": "fc.view.ShareRecPage"
        },
        "Trial": {
            "url": "fc-layout/view/TrialPage.scene",
            "script": "fc.view.TrialPage"
        },
        "Turntable": {
            "url": "fc-layout/view/TurntablePage.scene",
            "script": "fc.view.TurntablePage"
        },
        "InsertAd": {
            "url": "fc-layout/view/VideoPage.scene",
            "script": "fc.view.InsertAdPage"
        },
        "FullScreenAd": {
            "url": "fc-layout/view/VideoPage.scene",
            "script": "fc.view.FullScreenAdPage"
        },
        "NativeAdb": {
            "url": "fc-layout/view/NativeAdbPage.scene",
            "script": "fc.view.NativeAdPage"
        },
        "NativeAdm": {
            "url": "fc-layout/view/NativeAdmPage.scene",
            "script": "fc.view.NativeAdPage"
        }
    };    
    var sceneCfg = {
        "loading": {
            "type": "sync",
            "array": [
                {
                    "viewId": "Loading"
                }
            ]
        },
        "home": {
            "type": "sync",
            "array": [
                {
                    "viewId": "InsertAd",
                    "checkCode": 3,
                    "zOrder": 2
                },
                {
                    "viewId": "Home",
                    "zOrder": 1
                },
                {
                    "viewId": "Sign",
                    "checkCode": 1,
                    "zOrder": 2
                },
                {
                    "viewId": "Store",
                    "checkCode": 2,
                    "zOrder": 2,
                    "delay": 200
                }
            ]
        },
        "prepare": {
            "type": "sequence",
            "array": [
                {
                    "viewId": "Trial",
                    "checkCode": 4
                }
            ]
        },
        "game": {
            "type": "sync",
            "array": [
                {
                    "viewId": "Game"
                }
            ]
        },
        "success": {
            "type": "sequence",
            "array": [
                {
                    "viewId": "InsertAd"
                },
                {
                    "viewId": "SecretAward"
                },
                {
                    "viewId": "Success"
                }
            ]
        },
        "failed": {
            "type": "sequence",
            "array": [
                {
                    "viewId": "InsertAd"
                },
                {
                    "viewId": "SecretBox"
                },
                {
                    "viewId": "Failed"
                }
            ]
        },
        "over": {
            "type": "sequence",
            "array": [
            ]
        }
    };    
    var bannerConfig = {
        "Sign": {"delay": 10},
        "Trial": {"delay": 10},
        "Failed": {"delay": 10},
        "Success": {"delay": 10},
        "Home": {"delay": 10},
        "SecretBox": {"delay": 10},
        "SecretAward": {"delay": 10}
    };
    var additiveUIConfig = {
        "Home": [
            {
                "viewUrl": "fc-layout/prefab/additive/GoldShowAdd.json",
                "parentRoot":"topUI",
                "x":32,
                "y":160,
                "scriptType":"fc.view.GoldShowAdd"
            },
            {
                "viewUrl": "fc-layout/prefab/additive/StrengthShowAdd.json",
                "parentRoot":"topUI",
                "x":310,
                "y":160,
                "scriptType":"fc.view.StrengthShowAdd",
                "checkCode": 2
            },
            {
                "viewUrl": "fc-layout/prefab/additive/StartGame.json",
                "parentRoot":"bottomUI",
                "x":375,
                "y":359,
                "scriptType":"fc.view.AniLoopPlay"
            }
        ],
        "Game": [
            {
                "viewUrl": "fc-layout/prefab/additive/ResetBtn.json",
                "parentRoot":"topUI",
                "x":30,
                "y":420,
                "scriptType":"fc.view.ResetBtn",
                "checkCode": 3
            },
            {
                "viewUrl": "fc-layout/prefab/additive/GoldShow.json",
                "parentRoot":"topUI",
                "x":32,
                "y":160,
                "scriptType":"fc.view.GoldShow"
            },
            {
                "viewUrl": "fc-layout/prefab/additive/StrengthShow.json",
                "parentRoot":"topUI",
                "x":310,
                "y":160,
                "scriptType":"fc.view.StrengthShow",
                "checkCode": 2
            }
        ],
        "Failed": [
            {
                "viewUrl": "fc-layout/prefab/additive/FailEmoji.json",
                "parentRoot": "middleUI",
                "x": 375,
                "y": 120,
                "scriptType": "fc.view.AniLoopPlay"
            }
        ],
        "Success": [
            {
                "viewUrl": "fc-layout/prefab/additive/SuccessEmoji.json",
                "parentRoot": "middleUI",
                "x": 375,
                "y": 120,
                "scriptType": "fc.view.AniLoopPlay"
            }
        ]
    };
    var playerCfg = {
        "userId": "1",
        "isNew": 1, 
        "isInGuide": 1,
        "signedDays": 0, 
        "signTime": 0,
        "levelId": 1,
        "gold": 0,
        "strength": 5,
        "sound": 1,
        "goodsInUse": [0],
        "goodsInBag": [[0]]
    };

    class PlatformConfig extends Laya.Script {

        static get playerInfo() {
            return GameFrame.playerInfo;
        }

        static set playerInfo(val) {
            if (val) {
                var keys = Object.keys(val);
                for (var index = 0; index < keys.length; index++) {
                    GameFrame.playerInfo[keys[index]] = val[keys[index]];
                }
                if (keys.length > 0) {
                    GameFrame.savePlayerInfo(val);
                }
            }
        }

        static init(appCfgUrl) {
            Laya.stage.addComponent(PlatformConfig);
            Laya.stage.addComponent(BannerCtrl);
            Laya.stage.addComponent(AdditiveUICtrl);
            Laya.stage.addComponent(ActivityCenter);
            GameFrame.startUp(appCfgUrl ? appCfgUrl : `config/${this.platformTag}/appConfig.${this.platformTag}.json`, PlatformConfig.platformTag, viewCfg, sceneCfg, playerCfg);
        }

        constructor() {
            super();
        }

        onAwake() {
            GameFrame.setViewValidCheckFun(isViewValid);
            BannerCtrl.bannerConfig = bannerConfig;
            AdditiveUICtrl.additiveUIConfig = additiveUIConfig;
            AdditiveUICtrl.checkFun = isAdditiveUIVisible;
            Laya.stage.once(Event.LAUNCH_COMPLETED, this, this.onLaunchResReady);
            Laya.stage.on(Event.UI_SCENE_LOADED, this, this.onSceneLoaded);
            Laya.stage.on(Event.UI_SCENE_UNLOADED, this, this.onSceneUnloaded);
        }

        onDestroy() {
            Laya.stage.off(Event.UI_SCENE_LOADED, this, this.onSceneLoaded);
            Laya.stage.off(Event.UI_SCENE_UNLOADED, this, this.onSceneUnloaded);
        }

        onLaunchResReady(scene3d) {
            var pluginCfg = GameFrame.pluginCfg;
            if (pluginCfg) {
                if (pluginCfg.pg_share_opened == null) {
                    pluginCfg.pg_share_opened = 0;
                }
                GameFrame.pluginCfg.pg_video_style = pluginCfg.pg_video_style || [0,1,3,4];
            }
        }

        onSceneUnloaded(sceneId) {
            if (sceneId == SceneId.Game) {
                Audio.stopMusic();
            }
        }

        onSceneLoaded(sceneId) {
            if (sceneId == SceneId.Game) {
                if (GameFrame.appInfo.bgm) {
                    Audio.playMusic(GameFrame.appInfo.bgm);
                }
            }
        }
    }
    PlatformConfig.platformTag = "ssjjh5";
    
    Config.isAntialias = true;
    Config.useWebGL2 = false;
    exports.PlatformConfig = PlatformConfig;
    
}(window.fc.platform, Laya));